#include <u.h>
#include <libc.h>
#include <draw.h>
#include <event.h>
#include <keyboard.h>
#include "taskbar.h"

static char* tip={"%s\nType: %s\nSize: %d\nOwner[%s] %s\nGroup[%s] %s\nOther %s\n"};
static Dir* Dirs=nil;
static Image *co_folder,*co_file,*co_exe,*co_tip;
static char CurPath[256];
static int CurPos=0,MaxDir,MaxView;
static Rectangle items[1024];
static Point pxy;
static int tip_draw=0,cnt_time=0,maxx,maxy;

void mkperm(char *str,int perm)
{
	//rwx
	if (perm & DMREAD)
		str[0]='r';
	else
		str[0]='-';

	if (perm & DMWRITE)
		str[1]='w';
	else
		str[1]='-';

	if (perm & DMEXEC)
		str[2]='x';
	else
		str[2]='-';

	str[3]=0;
}

void DrawTip(int id,Point xy)
{
	char Result[1024];
	char Path[256],pu[4],pg[4],po[4];
	int size,cx,cy,len,i,ccx;
	Point p;
	Rectangle r;

	strcpy(Path,CurPath);
	if (Path[strlen(Path)-1]!='/')
		strcat(Path,"/");
	strcat(Path,Dirs[id].name);

	mkperm(pu,Dirs[id].mode);			//Owner (User)
	mkperm(pg,Dirs[id].mode<<3);		//Group
	mkperm(po,Dirs[id].mode<<6);		//Other

	if (Dirs[id].mode & DMDIR)
	{
		size=Dirs[id].length;
		sprint(Result,tip,Path,"Dir",size,Dirs[id].uid,pu,Dirs[id].gid,pg,po);
	}
	else
	{
		size=Dirs[id].length;
		sprint(Result,tip,Path,"File",size,Dirs[id].uid,pu,Dirs[id].gid,pg,po);
	}

	//Size of rect
	len=strlen(Result);	cx=0;	cy=0;	ccx=0;
	for (i=0;i<len;i++)
	{
		if (Result[i]=='\n')
		{
			cy++;
			if (ccx>cx)
				cx=ccx;
		
			ccx=0;
		}
		else
			ccx++;
	}

	cx=cx*pxps;
	cy*=font->height;

	r.min.x=xy.x;	r.min.y=xy.y;	r.max.x=xy.x+cx;	r.max.y=xy.y+cy;
	draw(screen,r,co_tip,display->white,ZP);
	border(screen,r,1,display->black,ZP);

	//Text output
	p.x=xy.x+text_margin;	p.y=xy.y;	ccx=0;
	for (i=0;i<len;i++)
	{
		if (Result[i]=='\n')
		{
			stringn(screen,p,display->black,ZP,font,&Result[ccx],i-ccx);
			p.y+=font->height;
			ccx=++i;
		}
	}
}

int GetID(Point xy)
{
	int i;
	for (i=0;i<MaxView;i++)
		if (ptinrect(xy,items[i]))
			return (i+CurPos);
	return -1;
}

void mgr_release()
{
	freeimage(co_folder);
	freeimage(co_file);
	freeimage(co_exe);
	freeimage(co_tip);
	free(Dirs);
}

void mgr_eresized(Rectangle r)
{
	Rectangle rect;
	int i,cw,width,height,x,y;
	
	if (CurPos>MaxDir)
		CurPos=0;

	draw(screen,r,co_back,display->opaque,ZP);

	width=0;		//Find max width
	for (i=0;i<MaxDir;i++)
	{
		cw=pxps*strlen(Dirs[i].name);
		if (cw>width) width=cw;
	}

	height=font->height;

	maxx=(r.max.x-r.min.x)/(width+item_margin_x);
	maxy=(r.max.y-r.min.y)/(height+item_margin_y);

	if (maxx<1) maxx=1;	//Show at least one item
	if (maxy<1) maxy=1;

	MaxView=0;
	for (y=0;y<maxy;y++)
	for (x=0;x<maxx;x++)
	{
		if ((MaxView+CurPos)>=MaxDir)
			return;

		rect.min.x=r.min.x+x*(width+item_margin_x);
		rect.min.y=r.min.y+y*(font->height+item_margin_y);
		rect.max.x=rect.min.x+width;
		rect.max.y=rect.min.y+height;

		border(screen,rect,1,display->black,ZP);
		items[MaxView]=rect;
		rect.min.x+=text_margin;

		if (Dirs[CurPos+MaxView].mode & DMDIR)
			string(screen,rect.min,co_folder,ZP,font,Dirs[CurPos+MaxView].name);
		else
			if (Dirs[CurPos+MaxView].mode & DMEXEC)
				string(screen,rect.min,co_exe,ZP,font,Dirs[CurPos+MaxView].name);
			else
				string(screen,rect.min,co_file,ZP,font,Dirs[CurPos+MaxView].name);

		MaxView++;
	}
}

void SortDir(void)
{
	int i,dirty;
	Dir copy;

	do {
		dirty=0;
		for (i=1;i<MaxDir;i++)
			if (strcmp(Dirs[i-1].name,Dirs[i].name)>0)
			{
				copy=Dirs[i-1];
				Dirs[i-1]=Dirs[i];
				Dirs[i]=copy;
				dirty=1;
			}
	}
	while(dirty);
}

int ReloadDir(void)
{
	int fd;

	if ((fd=open(CurPath,OREAD))<0)
		return 0;
	
	free(Dirs);
	Dirs=nil;

	MaxDir=dirreadall(fd,&Dirs);
	close(fd);

	SortDir();

	return 1;
}

int cddotdot(void)
{
	int len,c;

	if ((len=strlen(CurPath))==1)
		return 0;

	for (c=len;c>=0;c--)
		if ((CurPath[c]=='/'))
		{
			if (c==0)
				CurPath[c+1]=0;	else		CurPath[c]=0;

			break;
		}

	return ReloadDir();
}

int cdthis(char* folder)
{		
	if (folder!=0)
	{
		if (CurPath[strlen(CurPath)-1]!='/')
			strcat(CurPath,"/");

		strcat(CurPath,folder);
	}

	return ReloadDir();
}

void Open(int id)
{
	char Run[1024];

	strcpy(Run,CurPath);
	if (Run[strlen(Run)-1]!='/')
		strcat(Run,"/");
	strcat(Run,Dirs[id].name);

	OpenFile(Run);
}

int mgr_init(Mode *m)
{
	strcpy(CurPath,"/");

	if (!cdthis(0))
		return 0;

	if ((co_folder=allocimage(display,Rect(0,0,1,1),RGB24,1,DMedblue))<=(Image*)0)
		return 0;

	if ((co_file=allocimage(display,Rect(0,0,1,1),RGB24,1,DBlack))<=(Image*)0)
	{
		freeimage(co_folder);
		return 0;
	}

	if ((co_exe=allocimage(display,Rect(0,0,1,1),RGB24,1,DRed))<=(Image*)0)
	{
		freeimage(co_folder);
		freeimage(co_file);
		return 0;
	}

	if ((co_tip=allocimage(display,Rect(0,0,1,1),RGB24,1,DCyan))<=(Image*)0)
	{
		freeimage(co_folder);
		freeimage(co_file);
		freeimage(co_exe);
		return 0;
	}

	return FLAG_MENU|FLAG_READY;
}

int mgr_event(Event e,int type)
{
	int id;
	Mouse m;

	switch(type)
	{
		case Ekeyboard:
			switch(e.kbdc)
			{
				case Kup:
					if (MaxDir<=maxx*maxy)
						break;

					if ((CurPos-=maxx)<0)
						if ((CurPos=MaxDir-maxx*maxy)<0)
							CurPos=0;
					UpdateWindow();
					break;
				case Kdown:
					if (MaxDir<=maxx*maxy)
						break;

					if ((CurPos+=maxx)>MaxDir)
						CurPos=0;
					UpdateWindow();
					break;
				case Khome:
					if (MaxDir<=maxx*maxy)
						break;

					CurPos=0;
					UpdateWindow();
					break;
				case Kend:
					if (MaxDir<=maxx*maxy)
						break;

					if ((CurPos=MaxDir-maxx*maxy)<0)
						CurPos=0;
					UpdateWindow();
					break;
			}
			break;
		case Emouse:
//			if ((pxy.x!=e.mouse.xy.x) || (pxy.y!=e.mouse.xy.y))
//			{
//				cnt_time=0;
//				if (tip_draw)
//				{
//					tip_draw=0;
//					UpdateWindow();
//				}
//				pxy=e.mouse.xy;
//			}

			switch(e.mouse.buttons)
			{
				case 1:
					m=emouse();
					if (m.buttons!=0)
						break;

					if ((m.xy.x!=e.mouse.xy.x) || (m.xy.y!=e.mouse.xy.y))
						break;

					if ((id=GetID(e.mouse.xy))>=0)
						if (Dirs[id].mode & DMDIR)
						{
							if (cdthis(Dirs[id].name))
								UpdateWindow();
						}
						else
						{
							Open(id);
						}
					 break;
				case 2:
					m=emouse();
					if (m.buttons!=0)
						break;

					if (cddotdot())
						UpdateWindow();
					break;
				case 8:
					if (MaxDir<=maxx*maxy)
						break;

					if ((CurPos-=maxx)<0)
						if ((CurPos=MaxDir-maxx*maxy)<0)
							CurPos=0;
					UpdateWindow();
					break;
				case 16:
					if (MaxDir<=maxx*maxy)
						break;

					if ((CurPos+=maxx)>MaxDir)
						CurPos=0;
					UpdateWindow();
					break;
			}
			break;
//		default:
//			if (type==timer)
//			{
//				cnt_time++;
//				if ((cnt_time>tip_time) && (tip_draw==0))
//					if ((id=GetID(pxy))>=0)
//					{
//						DrawTip(id,pxy);
//						tip_draw=1;
//					}
//			}
//			break;
//			Disable tooltip because it not useful in little file manager...

	}
	return 1;
}

