#include <u.h>
#include <libc.h>
#include <draw.h>
#include <event.h>
#include <keyboard.h>
#include "taskbar.h"
#include "kbmap.h"

#define MAX_NAME 50
#define MAX_ITEM 50

typedef struct kbitem kbitem;
struct kbitem
{
	Rectangle r;
	char name[MAX_NAME];
	int current;
};

static kbitem items[MAX_ITEM];
static int MaxItem=0,MaxView=0;
static char buf[128];
static Image *co_cur;

static void RefreshItems(void)
{
	Dir *Dirs;
	int fd,i,max;

	if ((fd=open("/sys/lib/kbmap",OREAD))<0)
		return;

	MaxItem=0;
	if ((max=dirreadall(fd,&Dirs))<=0)
		return;

	for (i=0;i<max;i++)
	if ((Dirs[i].mode & DMDIR)==0)
	{
		strncpy(items[MaxItem].name,Dirs[i].name,MAX_NAME);
		items[MaxItem].current=0;
		MaxItem++;
	}

	close(fd);
}

int kb_init(Mode *m)
{
	RefreshItems();
	co_cur=allocimage(display,Rect(0,0,1,1),CMAP8,1,DMagenta);
	return FLAG_MENU|FLAG_READY;
}

void kb_release()
{
	freeimage(co_cur);
}

void kb_eresized(Rectangle r)
{
	int i,height,width,x,y,maxx,maxy;
	Rectangle rect;

	draw(screen,r,co_back,display->opaque,ZP);

	height=font->height;
	maxy=(r.max.y-r.min.y)/(height+item_margin_y);
	if (maxy<1) maxy=1;
	maxx=MaxItem/maxy+1;
	width=(r.max.x-r.min.x)/maxx-item_margin_x;

	MaxView=0;
	for (y=0;y<maxy;y++)
	for (x=0;x<maxx;x++)
	{
		if (MaxView>=MaxItem)
			return;

		rect.min.x=r.min.x+x*(width+item_margin_x);
		rect.min.y=r.min.y+y*(height+item_margin_y);
		rect.max.x=rect.min.x+width;
		rect.max.y=rect.min.y+height;
		items[MaxView].r=rect;

		border(screen,rect,1,display->black,ZP);

		rect.min.x+=text_margin;
		
		if (items[MaxView].current)
			string(screen,rect.min,co_cur,ZP,font,items[MaxView].name);
		else
			string(screen,rect.min,display->black,ZP,font,items[MaxView].name);

		MaxView++;
	}
}

void LoadKbMap(int i)
{
	int fd,fd2,n;
	char data[1024];

	sprint(buf,"/sys/lib/kbmap/%s",items[i].name);
	if ((fd=open(buf,OREAD))<0)
		return;

	if ((fd2=open("/dev/kbmap",OWRITE))<0)
	{
		close(fd);
		return;
	}

	while((n=read(fd,&data,1024)))
		write(fd2,&data,n);
		
	for (n=0;n<MaxItem;n++)
		items[n].current=0;
	items[i].current=1;	

	close(fd2);
	close(fd);
}

int kb_event(Event e,int type)
{
	Mouse m;
	int i;

	switch(type)
	{
		case Emouse:
			switch(e.mouse.buttons)
			{
				case 1:
					m=emouse();
					if (m.buttons!=0)
						return 1;

					if ((m.xy.x!=e.mouse.xy.x) || (m.xy.y!=e.mouse.xy.y))
						return 1;

					for (i=0;i<MaxView;i++)
						if (ptinrect(e.mouse.xy,items[i].r))
						{
							LoadKbMap(i);
							UpdateWindow();
							break;
						}
					break;
			}
			break;
	}
	return 1;
}

