#include <u.h>
#include <libc.h>
#include <draw.h>
#include <event.h>
#include <keyboard.h>
#include "taskbar.h"
#include "fmgr.h"
#include "start.h"
#include "window.h"
#include "kbmap.h"

static Mode pm[]={
	{"Windows manager",0,0,window_eresized,window_event,window_release,window_init},
	{"File browser",0,0,mgr_eresized,mgr_event,mgr_release,mgr_init},
	{"Start menu",0,0,start_eresized,start_event,start_release,start_init},
	{"Kbmap",0,0,kb_eresized,kb_event,kb_release,kb_init}
};

#define MODE_LAST sizeof(pm)/sizeof(Mode)

Image *co_back;
static int mode=-1;
int timer;

int OpenFile(char* FileName)
{
	Dir *dir;
	int exec;

	if ((dir=dirstat(FileName))==nil)
		return 0;
	if (dir->mode & DMEXEC)
		exec=1;
	else
		exec=0;
	free(dir);
	

	switch(rfork(RFPROC|RFNOWAIT|RFCFDG|RFNAMEG|RFENVG))
	{
		case -1:
			return 0;
		case 0:
			newwindow("");
			if (exec)
			{
				execl("/bin/rc","rc","-c",FileName,0);
			}
			else
				execl("/bin/plumb","plumb",FileName,0);
			break;
		default:
			break;
	}

	return 1;
}

void UpdateWindow(void)
{
	Rectangle r;
	int i;

	lockdisplay(display);
	draw(screen,screen->r,co_back,display->opaque,ZP);

	r.min.x=screen->r.min.x+x_margin;
	r.min.y=screen->r.min.y+y_margin;
	r.max.x=r.min.x+app_zone;
	r.max.y=screen->r.max.y-y_margin;
	border(screen,r,1,display->black,ZP);

	r.min.x=screen->r.min.x+2*x_margin+app_zone;
	r.min.y=screen->r.min.y+y_margin;
	r.max.y=screen->r.max.y-y_margin;
	r.max.x=screen->r.max.x-x_margin;

	pm[mode].resize_proc(r);

	flushimage(display,1);
	unlockdisplay(display);
}

void eresized(int new)
{
	if(getwindow(display, Refnone) < 0)
		sysfatal("resize failed: %r");
	else
		UpdateWindow();
}

void main(int argc,char** argv)
{
	Event e;
	int type,i,max_mode=0;
	char* MenuStr[MODE_LAST+2];
	char* ExitStr={"Exit"};
	Menu menu={MenuStr,0,0};

	initdraw(0,0,"Taskbar");
	einit(Emouse|Ekeyboard);
	timer=etimer(0,1000);

	display->locking=1;
	unlockdisplay(display);

	co_back=allocimage(display,Rect(0,0,1,1),CMAP8,1,DPaleyellow);

	for (i=0;i<MODE_LAST;i++)
		pm[i].flag=pm[i].init_proc(&pm[i]);

	for (i=0;i<MODE_LAST;i++)
		if ((pm[i].flag & FLAG_READY) && (pm[i].flag & FLAG_MENU))
		{
			mode=i;
			break;
		}

	if (mode<0)
		exits(0);

	for (i=0;i<MODE_LAST;i++)
		if ((pm[i].flag & FLAG_READY) && (pm[i].flag & FLAG_MENU))
		{
			MenuStr[max_mode]=pm[i].Name;
			max_mode++;
		}
	MenuStr[max_mode]=ExitStr;
	MenuStr[max_mode+1]=0;

	eresized(0);

	while(1)
	{
		type=event(&e);

		if ((type==Emouse) && (e.mouse.xy.x<screen->r.min.x+app_zone))
			switch(e.mouse.buttons)
			{
			case 1:
				if ((i=emenuhit(1,&e.mouse,&menu))>=0)
				{
					if (i==max_mode)
					{
						for (i=0;i<MODE_LAST;i++)
							if (pm[i].flag & FLAG_READY)
								pm[i].release_proc();
						freeimage(co_back);
						exits(0);
					}

					mode=i;
					eresized(0);
					continue;
				}
				break;
			
			}


		pm[mode].event_proc(e,type);
	}
}
