#include <u.h>
#include <libc.h>
#include <draw.h>
#include <event.h>
#include <keyboard.h>
#include "taskbar.h"
#include "start.h"

#define MAX_NAME	100
#define MAX_PATH 256
#define MAX_ITEM 100

typedef struct Start Start;
struct Start
{
	char Name[MAX_NAME];
	char Bin[MAX_PATH];
	int Flag;
};

static Start StartMenu[MAX_ITEM];
static int MenuPos=-1,CurPos=0,MaxMenu=0,MaxView=0,width=0,maxx;
static Rectangle items[100];
static Image *co_item,*co_tip,*co_folder;

int start_init(Mode *m)
{
	char FileName[1024],sym,*env,WorkStr[1024];
	int fd,i,len;

	if ((env=getenv("home"))==0)
		return 0;

	strcpy(FileName,env);
	free(env);

	if (FileName[strlen(FileName)-1]!='/')
		strcat(FileName,"/");
	strcat(FileName,"lib/startmenu");

	if ((fd=open(FileName,OREAD))>=0)
	{
		WorkStr[0]=0;
		while(read(fd,&sym,1))
		{
			switch(sym)
			{
				case '\n':
					len=strlen(WorkStr);

					switch(WorkStr[0])
					{
						case 0:	//Skip free lines
							break;
						case '\t':
							for (i=0;i<len;i++)
							{
								if (WorkStr[i]=='\\')
								{
									strncpy(StartMenu[MaxMenu].Name,&WorkStr[1],i-1);
									strcpy(StartMenu[MaxMenu].Bin,&WorkStr[i+1]);
									StartMenu[MaxMenu].Flag=FLAG_ITEM;
									if (pxps*strlen(StartMenu[MaxMenu].Name)>width)
										width=pxps*strlen(StartMenu[MaxMenu].Name);
									MaxMenu++;
								}
							}
							break;
						default:
							strncpy(StartMenu[MaxMenu].Name,WorkStr,MAX_NAME);
							StartMenu[MaxMenu].Flag=FLAG_SUBFOLDER;
							if (pxps*strlen(StartMenu[MaxMenu].Name)>width)
								width=pxps*strlen(StartMenu[MaxMenu].Name);
							MaxMenu++;
							break;
					}
					WorkStr[0]=0;
					break;
				case '#':
					do 	read(fd,&sym,1);	while(sym!='\n');
					continue;
				default:
					strncat(WorkStr,&sym,1);
					break;
			}
		}
		close(fd);
	}

	if ((co_item=allocimage(display,Rect(0,0,1,1),RGB24,1,DMagenta))<=(Image*)0)
		return 0;

	if ((co_folder=allocimage(display,Rect(0,0,1,1),RGB24,1,DMagenta))<=(Image*)0)
	{
		freeimage(co_item);
		return 0;
	}

	return FLAG_MENU|FLAG_READY;
}

void start_release()
{
	freeimage(co_folder);
	freeimage(co_item);
}

void start_eresized(Rectangle r)
{
	int i,height,maxy,x,y;
	Rectangle rect;

	draw(screen,r,co_back,display->opaque,ZP);
	height=font->height;

	maxx=(r.max.x-r.min.x)/(width+item_margin_x);
	maxy=(r.max.y-r.min.y)/(height+item_margin_y);

	if (maxx<1) maxx=1;	//Show at least one item
	if (maxy<1) maxy=1;

	MaxView=0;	i=0;
	for (y=0;y<maxy;y++)
	for (x=0;x<maxx;x++)
	{

cucle:

		if (MenuPos<0)
		{
			if (i+CurPos>=MaxMenu)
				return;
		}
		else
		{
			if (i+CurPos+MenuPos>=MaxMenu)
				return;
		}

		rect.min.x=r.min.x+x*(width+item_margin_x);
		rect.max.x=rect.min.x+width;
		rect.min.y=r.min.y+y*(height+item_margin_y);
		rect.max.y=rect.min.y+height;
		items[MaxView]=rect;

		if (MenuPos<0)
		{
			if (StartMenu[i].Flag & FLAG_SUBFOLDER)
			{
				border(screen,rect,1,display->black,ZP);
				rect.min.x+=text_margin;
				string(screen,rect.min,co_folder,ZP,font,StartMenu[i].Name);
				MaxView++;	i++;
				continue;
			}
			
			i++;
		}
		else
		{
			if (StartMenu[i+CurPos+MenuPos].Flag & FLAG_SUBFOLDER)
				return;

			border(screen,rect,1,display->black,ZP);
			rect.min.x+=text_margin;
			string(screen,rect.min,co_item,ZP,font,StartMenu[i+CurPos+MenuPos].Name);
			MaxView++;	i++;
			continue;
		}
		
		goto cucle;
	}
}

int start_event(Event e,int type)
{
	Mouse m;
	int i,c;

	switch(type)
	{
		case Ekeyboard:
			switch(e.kbdc)
			{
				case Khome:
					CurPos=0;
					UpdateWindow();
					break;
				case Kend:
					if ((CurPos-=MaxView)<0)
						CurPos=0;
					UpdateWindow();
					break;
				case Kup:
				case Kpgup:
					if ((CurPos-=maxx)<0)
						if ((CurPos=MaxView-maxx)<0)
							CurPos=0;
					UpdateWindow();
					break;
				case Kdown:
				case Kpgdown:
					if ((CurPos+=maxx)>=MaxView)
						CurPos=0;
					UpdateWindow();
					break;
			}
			break;
		case Emouse:
			switch(e.mouse.buttons)
			{
				case 1:
					m=emouse();
					if (m.buttons!=0)
						break;

					if ((m.xy.x!=e.mouse.xy.x) || (m.xy.y!=e.mouse.xy.y))
						break;	

					for (i=0;i<MaxView;i++)
						if (ptinrect(e.mouse.xy,items[i]))
						{
							if (MenuPos<0)
							{
								for (c=0;c<MaxMenu;c++)
									if (StartMenu[c+CurPos].Flag & FLAG_SUBFOLDER)
										if (--i<0)
										{
											MenuPos=CurPos+c+1;	
											CurPos=0;
											UpdateWindow();
											break;
										}
							}
							else
							{
								if (StartMenu[i+CurPos+MenuPos].Flag & FLAG_ITEM)
								{
									OpenFile(StartMenu[i+CurPos+MenuPos].Bin);
								}
							}
							break;
						}
					 break;
				case 2:
					m=emouse();
					if (m.buttons!=0)
						break;

					MenuPos=-1;
					UpdateWindow();
					break;
				case 8:
					if ((CurPos-=maxx)<0)
						if ((CurPos=MaxView-maxx)<0)
							CurPos=0;
					UpdateWindow();
					break;
				case 16:
					if ((CurPos+=maxx)>=MaxView)
						CurPos=0;
					UpdateWindow();
					break;
			}
			break;
		default:
			break;
	}
	return 1;
}

