#include <u.h>
#include <libc.h>
#include <draw.h>
#include <event.h>
#include <keyboard.h>
#include "taskbar.h"
#include "window.h"

#define MAX_NAME 100
#define MAX_ITEM 100

typedef struct WndItem WndItem;
struct WndItem
{
	int n;
	char Name[MAX_NAME+1];
	Rectangle r;
};

static WndItem items[MAX_ITEM];
static int width,MaxView=0,MaxItem=0,CurPos=0,maxx,maxy;
static char buf[256];

void RefreshItems(void)
{
	int fd,cnt,cfd,i;
	Dir *dir;

	MaxItem=0;
	width=0;

	if ((fd=open("/dev/wsys",OREAD))<0)
		return;

	cnt=dirreadall(fd,&dir);

	if (cnt<0)
		return;

	for (i=0;i<cnt;i++)
	{
		sprint(buf,"/dev/wsys/%s/label",dir[i].name);
		if ((cfd=open(buf,OREAD))<0)
			continue;
	
		items[MaxItem].n=atoi(dir[i].name);

		items[MaxItem].Name[read(cfd,items[MaxItem].Name,MAX_NAME)]=0;
		if (pxps*strlen(items[MaxItem].Name)>width)
			width=pxps*strlen(items[MaxItem].Name);
		close(cfd);

		MaxItem++;
	}

	free(dir);
	close(fd);
}

int window_init(Mode *m)
{
	RefreshItems();
	return FLAG_MENU|FLAG_READY;
}

void window_release()
{
	
}

void window_eresized(Rectangle r)
{
	int i,height,x,y;
	Rectangle rect;

	draw(screen,r,co_back,display->opaque,ZP);

	height=font->height;

	maxx=(r.max.x-r.min.x)/(width+item_margin_x);
	maxy=(r.max.y-r.min.y)/(height+item_margin_y);
	
	if (maxx<1) maxx=1;	//Show at least one item
	if (maxy<1) maxy=1;

	MaxView=0;
	for (y=0;y<maxy;y++)
	for (x=0;x<maxx;x++)
	{
		if (MaxView+CurPos>=MaxItem)
			return;

		rect.min.x=r.min.x+x*(width+item_margin_x);
		rect.min.y=r.min.y+y*(height+item_margin_y);
		rect.max.x=rect.min.x+width;
		rect.max.y=rect.min.y+height;
		items[MaxView+CurPos].r=rect;

		border(screen,rect,1,display->black,ZP);

		rect.min.x+=text_margin;
		string(screen,rect.min,display->black,ZP,font,items[MaxView+CurPos].Name);

		MaxView++;
	}
}

int window_event(Event e,int type)
{
	Mouse m;
	int i,fd;

	switch(type)
	{
		case Ekeyboard:
			switch(e.kbdc)
			{
				case Kup:
				case Kpgup:
					if (MaxItem<=maxx*maxy)
						return 1;

					if ((CurPos-=maxx)<0)
							CurPos=0;
					UpdateWindow();
					break;
				case Kdown:
				case Kpgdown:
					if (MaxItem<=maxx*maxy)
						return 1;

					if ((CurPos+=maxx)>MaxItem)
						CurPos=0;
					UpdateWindow();
					break;
				case Khome:
					CurPos=0;
					UpdateWindow();
					break;
				case Kend:
					if (MaxItem<=maxx*maxy)
						return 1;

					if ((CurPos=MaxItem-maxx*maxy)<0)
						CurPos=0;
					UpdateWindow();
					break;
			}
			break;
		case Emouse:
			switch(e.mouse.buttons)
			{
				case 1:
					m=emouse();
					if (m.buttons!=0)
						return 1;

					if ((m.xy.x!=e.mouse.xy.x) || (m.xy.y!=e.mouse.xy.y))
						return 1;

					for (i=CurPos;i<MaxView+CurPos;i++)
						if (ptinrect(e.mouse.xy,items[i].r))
						{
							sprint(buf,"/dev/wsys/%d/wctl",items[i].n);
							if ((fd=open(buf,OWRITE))<0)
								return 1;
							write(fd,"unhide\n",7);
							write(fd,"top\n",4);
							write(fd,"current\n",8);
							close(fd);
							break;
						}
					break;
				case 4:
					m=emouse();
					if (m.buttons!=0)
						return 1;

					if ((m.xy.x!=e.mouse.xy.x) || (m.xy.y!=e.mouse.xy.y))
						return 1;

					for (i=CurPos;i<MaxView+CurPos;i++)
						if (ptinrect(e.mouse.xy,items[i].r))
						{
							sprint(buf,"/dev/wsys/%d/wctl",items[i].n);
							if ((fd=open(buf,OWRITE))<0)
								return 1;
							write(fd,"hide\n",5);
							close(fd);
							return 1;
						}

					if ((fd=open("/dev/wctl",OWRITE))<0)
						return 1;
					write(fd,"new\n",4);
					close(fd);
			}
			break;
		default:
			if (type==timer)
			{
				RefreshItems();
				UpdateWindow();
			}
			break;
	}
	return 1;
}

